import { Component, OnInit } from '@angular/core';
import {StateService} from '../state.service';

@Component({
  selector: 'app-table-overview',
  templateUrl: './table-overview.component.html',
  styleUrls: ['./table-overview.component.css']
})
export class TableOverviewComponent implements OnInit {
  public subplots;
  public averages;
  constructor(private state: StateService) { }

  ngOnInit(): void {
    // this.state.onNewTable.subscribe(() => this.createHistograms());
    // this.state.onNewTableInfo.subscribe(() => this.createPrototypes());
  }

  createPrototypes(): void {
    this.averages = this.state.tableInfo.prototypes.map(prototype => {
      return {
        data: [
          {
            x: [...Array(prototype.average.length).keys()],
            y: prototype.average,
            type: 'line',
          },
          {
            x: [...Array(prototype.average.length).keys()],
            y: prototype.max,
            type: 'scatter',
            fill: null,
            mode: 'lines',
            line: {
              color: 'rgb(55, 128, 191)',
              width: 3
            }
          },
          {
            x: [...Array(prototype.average.length).keys()],
            y: prototype.min,
            type: 'scatter',
            fill: 'tonexty',
            mode: 'lines',
            line: {
              color: 'rgb(55, 128, 191)',
              width: 3
            }
          }
        ],
        layout: {
          showlegend: false,
          hovermode: 'closest',
          autosize: true,
          margin: {
            l: 10,
            r: 10,
            t: 10,
            b: 10,
            pad: 4
          },
          xaxis: {
            showticklabels: false
          },
          yaxis: {
            showticklabels: false
          },
          height: 100,
          width: screen.width * 0.1,
        }
      };
    });
  }

  async createHistograms() {
    this.subplots = [];
    this.averages = [];
    const table = this.state.table;
    this.subplots.push(
      {
        data: [{
          x: Object.keys(table),
          y: Object.values(table).map((values: number[]) => values.length), // / (this.service.rawValues.length - this.service.windowSize)),
          type: 'bar'
        }],
        layout: {
          hovermode: 'closest',
          autosize: true,
          margin: {
            l: 10,
            r: 10,
            t: 10,
            b: 10,
            pad: 4
          },
          xaxis: {
            showticklabels: false
          },
          yaxis: {
            showticklabels: false
          },
          height: 100,
          width: screen.width * 0.1,
        }
      }
    );
  }

  // async setQuery(data) {
  //   this.state.queryWindow = await this.state.getQueryWindow(data[0].y);
  //   await this.state.update();
  // }

  public get tables() {
    return this.state.table;
  }

  public get visible() {
    return !this.state.querySelectionMode;
  }
}
